/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/update/topology.h>
#include<vcg/complex/algorithms/update/normal.h>
using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f,
                                              vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::FFAdj, face::Normal3f,
                                      face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>,
                                       std::vector<MyFace> ,
                                       std::vector<MyEdge>  > {};


int main( int argc, char **argv )
{
  if(argc<3)
  {
    printf("Usage fgt_XXX <meshfilename.off> <perc>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  float percOfFace = atof(argv[2]);

  tri::RequirePerVertexNormal(m);
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  int numOfFaceToProcess = m.FN()*percOfFace;

  tri::Allocator<MyMesh>::CompactEveryVector(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  tri::UpdateNormal<MyMesh>::PerFaceNormalized(m);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  int boundaryCnt=0;
  float alpha = m.bbox.Diag()/30.0f;
  // Check for boundaries
  for(int i=0;i<numOfFaceToProcess;++i)
  {
    int faceInd = rand()%m.VN();

    Point3f barycenter = Barycenter(m.face[faceInd]);
    barycenter+=m.face[faceInd].N()*alpha;
    tri::Allocator<MyMesh>::AddVertex(m,barycenter);
  }

  tri::io::ExporterOFF<MyMesh>::Save(m,"pippo.off");

  return 0;
}
